#ifdef GL_ES
	#define LOWP lowp
	precision mediump float;
#else
	#define LOWP
#endif

uniform vec4 u_color;

uniform sampler2D u_texture;
varying vec2 v_texCoords;

varying vec3 normal, eyeVec;

const float PI = 3.1415;

uniform int u_time;

uniform float u_perc;

float time = float(u_time) / 50.0;


vec3 deform( in vec2 p )
{
    vec2 uv;

    vec2 q = vec2( sin(1.1*time+p.x),sin(1.2*time+p.y) );

    float a = atan(q.y,q.x);
    float r = sqrt(dot(q,q));

    uv.x = sin(0.0+1.0*time)+p.x*sqrt(r*r+1.0);
    uv.y = sin(0.6+1.1*time)+p.y*sqrt(r*r+1.0);

    return texture2D(u_texture,uv*.5).xyz;
}

vec4 getRadialBlurEffect(vec2 coords)
{
	vec4 res = vec4(0,0,0,0);
    vec2 p = -1.0 + 2.0 * coords;
    vec2 s = p;

    vec3 total = vec3(0.0);
    vec2 d = (vec2(0.0,0.0)-p)/40.0;
    float w = 1.0;
    for( int i=0; i<40; i++ )
    {
        vec3 res = deform(s);
        res = smoothstep(0.1,1.0,res*res);
        total += w*res;
        w *= .99;
        s += d;
    }
    total /= 40.0;
    float r = 1.5/(1.0+dot(p,p));
    res = vec4( total*r,1.0);
    
    return res;
}


//speed
const float speed = 0.01;
const float speed_x = 0.2;
const float speed_y = 0.2;

// geometry
const float intensity = 3.;
const int steps = 6;
const float frequency = 5.0;
const int angle = 7; // better when a prime

// reflection and emboss
const float delta = 20.0;
const float intence = 400.0;
const float emboss = 0.3;

//---------- crystals effect

  float col(vec2 coord)
  {
    float delta_theta = 2.0 * PI / float(angle);
    float col = 0.0;
    float theta = 0.0;
    for (int i = 0; i < steps; i++)
    {
      vec2 adjc = coord;
      theta = delta_theta*float(i);
      adjc.x += cos(theta)*time*speed + time * speed_x;
      adjc.y -= sin(theta)*time*speed - time * speed_y;
      col = col + cos( (adjc.x*cos(theta) - adjc.y*sin(theta))*frequency)*intensity;
    }

    return cos(col);
  }

//---------- main

vec4 getWaterEffect(vec2 coords) {
	vec4 res = vec4(0,0,0,0);
	float mult = 1.0;
	
	vec2 p = (coords.xy), c1 = p * mult, c2 = p * mult;
	float cc1 = col(c1);
	
	c2.x += mult / delta;
	float dx = emboss*(cc1-col(c2))/delta;
	
	c2.x = p.x;
	c2.y += mult / delta;
	float dy = emboss*(cc1-col(c2))/delta ;
	
	c1.x += dx;
	c1.y = mult -(c1.y+dy);
	
	c1 /= mult;
	
	float alpha = 1.0 + dot(dx,dy)*intence;
	res = texture2D(u_texture,c1)*(alpha);
	
	return res;
}

vec4 getTwistEffect(in vec2 coords) {
	vec4 res = vec4(0,0,0,0);
    vec2 p = -1.0 + 2.0 * coords.xy;
    vec2 uv;
   
    float a = atan(p.y,p.x);
    float r = sqrt(dot(p,p));

    uv.x = r - .25*time;
    uv.y = cos(a*5.0 + 2.0*sin(time+7.0*r)) ;

    vec3 col =  (.5+.5*uv.y)*texture2D(u_texture,uv).xyz;

    res = vec4(col,1.0);
    
    return res;
}



vec4 getWeirdEffect(in vec2 coords) {
	vec4 res = vec4(0,0,0,0);
	
	vec2 p = -1.0 + 2.0 * coords.xy;
    float a = time*40.0;
    float d,e,f,g=1.0/40.0,h,i,r,q;
    e=400.0*(p.x*0.5+0.5);
    f=400.0*(p.y*0.5+0.5);
    i=200.0+sin(e*g+a/150.0)*20.0;
    d=200.0+cos(f*g/2.0)*18.0+cos(e*g)*7.0;
    r=sqrt(pow(i-e,2.0)+pow(d-f,2.0));
    q=f/r;
    e=(r*cos(q))-a/2.0;f=(r*sin(q))-a/2.0;
    d=sin(e*g)*176.0+sin(e*g)*164.0+r;
    h=((f+d)+a/2.0)*g;
    i=cos(h+r*p.x/1.3)*(e+e+a)+cos(q*g*6.0)*(r+h/3.0);
    h=sin(f*g)*144.0-sin(e*g)*212.0*p.x;
    h=(h+(f-e)*q+sin(r-(a+h)/7.0)*10.0+i/4.0)*g;
    i+=cos(h*2.3*sin(a/350.0-q))*184.0*sin(q-(r*4.3+a/12.0)*g)+tan(r*g+h)*184.0*cos(r*g+h);
    i=mod(i/5.6,256.0)/64.0;
    if(i<0.0) i+=4.0;
    if(i>=2.0) i=4.0-i;
    d=r/350.0;
    d+=sin(d*d*8.0)*0.52;
    f=(sin(a*g)+1.0)/2.0;
    res = vec4(vec3(f*i/1.6,i/2.0+d/13.0,i)*d*p.x+vec3(i/1.3+d/8.0,i/2.0+d/18.0,i)*d*(1.0-p.x),1.0);
	
	return res;
}



vec4 getEvilEffect() {
	vec4 final_color = vec4(0.0, 0.0, 0.0, 1.0);

	vec3 N = normalize(normal);
	int i;
	for (i = 0; i < 1; ++i)
	{
		vec3 L = normalize(eyeVec);
		float lambertTerm = dot(N,L);
				
		if (lambertTerm > 0.0)
		{
			final_color += (vec4(0.3, 0.8, 0.8, 1.0) *  lambertTerm);

			vec3 E = normalize(eyeVec);
			vec3 R = reflect(-L, N);

			float shin = 5.0; 

			if(shin > 0.0)
			{
			    float specular = pow(max(dot(E,R), 0.0), shin);
			    final_color += (vec4(1.0, 1.0, 1.0, 1.0) * specular);
			}
			else
			{
				final_color += vec4(0.1, 0.1, 0.1, 1.0);
			}
   		}
	}

	final_color = 0.3 * (1.0 / final_color);
	return final_color;
}


vec4 getNightEffect() {
	vec4 final_color = vec4(0.0, 0.0, 0.0, 1.0);

	vec3 N = normalize(normal);
	int i;
	for (i = 0; i < 1; ++i)
	{
		vec3 L = normalize(eyeVec);
		float lambertTerm = dot(N,L);
				
		if (lambertTerm > 0.0)
		{
			final_color += (vec4(0.5, 1.0, 1.0, 1.0) *  lambertTerm);

			vec3 E = normalize(eyeVec);
			vec3 R = reflect(-L, N);

			float shin = 5.0; 

			if(shin > 0.0)
			{
			    float specular = pow(max(dot(E,R), 0.0), shin);
			    final_color += (vec4(1.0, 1.0, 1.0, 1.0) * specular);
			}
			else
			{
				final_color += vec4(0.1, 0.1, 0.1, 1.0);
			}
   		}
	}

	return final_color;
}


void main(void)
{
	float perc_a = 1.0 - u_perc;
	float perc_b = u_perc;

	gl_FragColor = (perc_a * getNightEffect() + getEvilEffect() * perc_b) * u_color * texture2D(u_texture, v_texCoords);
}
